"""
unittest-specs

A simple wrapper around the unittest package with different styles for specs
"""

__version__ = "1.1.1"
__author__ = "Phillip Goellner"


from unittest_specs.simple_test_spec import SimpleFlatSpec
from unittest_specs.fun_test_spec import describe, it, expect
